; R E P L A Y E R
;-------------------------------------------------------------------
;                                                 XL2S Entertainment

  module replay

  code

;--------------------------------------------------------------------

AYIO               = 010h
SCC_REGISTERS      = 0B800h 
SONG_VARS          = 0C000h   ; aligned 256 bytes
WAVE_BASE_ADRESS   = 4000h    ; aligned 256 bytes
SONG_START_ADDRESS = 4000h    ; aligned 256 bytes

SAMPLE_JUMP_TABLE  = SONG_VARS + 0C0h

@REPLAYPAGE        = 31       ; 256 kb ROM

;--------------------------------------------------------------------

  ; i n l c u d e  

  include _replay_variables.i

  include replay_scc_tone.i
  include replay_scc_wave.i
  include replay_scc_sample.i
  include replay_ay_tone.i
  include replay_ay_noise.i
  include replay_ay_am.i

  include replay_stream.i
  include replay_update.i


  code

; e n v i r o n m e n t   s e l e c t
;--------------------------------------------------------------------

set_song_environment
  ld a,(song_data_page)
  ld (05000h),a
  inc a
  ld (07000h),a
  inc a
  ld (09000h),a
  ret

set_wave_environment
  ld a,(wave_data_page)
  ld (05000h),a
  inc a
  ld (07000h),a
  ret

restore_game_environment
  ld hl,restorerompages
  ld a,(hl)
  inc l
  ld (05000h),a
  ld a,(hl)
  inc l
  ld (07000h),a
  ld a,(hl)
  inc l
  ld (09000h),a
  ret

  code

; s t a r t   s o n g
;--------------------------------------------------------------------
; hl -> song info

start_song
  ld a,(hl)                  ; wave data
  inc hl
  ld (wave_data_page),a
  ld e,(hl)
  inc hl
  ld d,(hl)
  inc hl
  ld (wave_data_base),de

  ld a,(hl)                  ; song data
  inc hl
  ld (song_data_page),a
  ld e,(hl)
  inc hl
  ld d,(hl)
  inc hl
  ld (song_data_base),de

  ld a,(hl)                  ; sample data
  inc hl
  or a
  jr z,.no_sample_jump_table
  ld e,(hl)
  inc hl
  ld d,(hl)
  ld (5000h),a               ; swap in page with sample table
  ex de,hl
  ld de,SAMPLE_JUMP_TABLE    ; copy table
  ld bc,16*4
  ldir
.no_sample_jump_table

  ld a,10111111b             ; init ay_reg 7: bit 7 & 6 = 10h
  ld (ay_reg_7),a

  ld (play_on),a             ; set play_on

  call set_song_environment

  ld hl,(song_data_base)       ; song start address
  call update_stream_pointers.init

  ld (tick_count),hl
  call process_stream_data

  jp restore_game_environment


; s t o p   s o n g
;--------------------------------------------------------------------

stop_song

  xor a
  ld (play_on),a       ; switch playback off
  call scc_stop
  call ay_stop
  ret


; p l a y   o u t   &   p r o c e s s
;--------------------------------------------------------------------

  ; u p d a t e   r e g i s t e r s 

play_out

  xor a
  ld hl,play_on
  cp (hl)
  ret z

  ; 5 0 Hz   e q u a l i s e r

  inc l                ; hl -> eq_on
  cp (hl)
  jr z,.skip_eq
  inc l                ; hl -> eq_tel
  inc (hl)
  ld a,(hl)
  cp 6
  ld a,0
  jr c,.skip_eq
  ld (hl),a
  ret
.skip_eq

  ; r e a d y  = >  u p d a t e   r e g i s t e r s

  ld l,low ready
  cp (hl)
  ret z
  ld (hl),a            ; clear ready flag
 
  call set_wave_environment

  call update_registers

  ld a,1
  ld (process),a       ; set flag for process song stream data

  jp restore_game_environment



  ; p r o c e s s   s o n g   d a t a

play_process

  xor a
  ld hl,process
  cp (hl)
  ret z
  ld (hl),a           ; clear process flag

  call set_song_environment

  call process_stream_data

  jp restore_game_environment



; i n i t   / c l e a r   v a r i a b e l e n
;--------------------------------------------------------------------

  ; S C C +

scc_init

  ld a,20h
  ld (0bffeh),a    ;scc+ mode in bank select mode

  ld a,REPLAYPAGE + 80h         ;scc active (page no. + 80h)
  ld (0b000h),a

  ld a,00011111b   ; switch all channels on
  ld (0B8AFh),a

  ret


scc_stop

  ld a,REPLAYPAGE + 80h     ;scc inschakelen
  ld (0b000h),a

  xor a            ;channels & volumes off
  ld b,6
  ld hl,0b8aah
1 ld (hl),a
  inc l
  djnz 1b

  ret


  ; A Y

ay_init

ay_stop
  ld a,7
  out (AYIO),a
  ld a,128+63
  out (AYIO+1),a     ; tone en noise off

  ld bc,3:AYIO     ; envelope & volumes off
  ld de,0:8
1 out (c),e
  inc e
  inc c
  out (c),d
  dec c
  djnz 1b
  ret


clear_variables
  xor a
  ld hl,SONG_VARS
  ld b,a
  call .clear_0
  inc h
  call .clear
  inc h
  call .clear
  inc h
  call .clear
  inc h
.clear
  ld b,32h
  ld l,a
.clear_0
1 ld (hl),a
  inc l
  djnz 1b
  ret

  endmodule

;eof
