; R E P L A Y E R
;-------------------------------------------------------------------
;                                                 XL2S Entertainment

; S C C    I S R   S A M P L E S
;--------------------------------------------------------------------
; in: h -> high sccv #
; uit:

process_isr_samples

  ; u p d a t e   t i m e r s

  ld hl,timers
  ld a,(hl)
  inc a
  cp 60
  jr c,1f
  xor a
1 ld (hl),a
  ld hl,timertab  
  add l
  ld l,a
  ld a,(hl)
  ld (isr_timer),a

  ; p r o c e s s   c h a n n e l s

  ld hl,sccv1+sccv.isrs_play               ; hl -> channel variables
  xor a
  ld (isr_samples_active),a                ; clear active status

  cp (hl)
  jr nz,.run1        ; process scc sample on channel 1
.nex2
  inc h
  cp (hl)
  jr nz,.run2        ; process scc sample on channel 2
.nex3
  inc h
  cp (hl)
  jr nz,.run3        ; process scc sample on channel 3
.nex4
  inc h
  cp (hl)
  jr nz,.run4        ; process scc sample on channel 4
.nex5
  inc h
  cp (hl)
  jr nz,.run5        ; process scc sample on channel 5
.nex6

  ld a,(song_data_page)
  ld (05000h),a
  inc a
  ld (07000h),a
  ret

.run1
  call .run
  jr .nex2
.run2
  call .run
  jr .nex3
.run3
  call .run
  jr .nex4
.run4
  call .run
  jr .nex5
.run5
  call .run
  jr .nex6



.first_trigger          ; => init timer from control byte
  inc l                 ; hl -> control
  ld a,(hl)
  and 7
  ld b,h                ; keep h
  ld hl,timermasktab
  add l 
  ld l,a
  ld a,(hl)             ; read timer mask
  ld h,b                ; restore h 

  ld l,sccv.isrs_period + 1   ; set value <> valid period range in high period
  ld (hl),l                   ; to ensure first time the period is always updated

  ld l,sccv.isrs_timer
  ld (hl),a             ; set mask
  jr .plays

.hold_sample
  ld l,sccv.isrs_play
  xor a
  ret


.run
  inc a
  ld (isr_samples_active),a                ; set active status

  inc l                 ; hl -> timer mask
  ld b,(hl)
  inc b
  jr z,.first_trigger
  dec b
  ld a,(isr_timer)
  and b
  jr z,.hold_sample

.plays
  inc l                ; hl -> control   
  ld c,(hl)            ; replay mode
  inc l
  ld a,(hl)            ; rom page no.
  inc l
  ld e,(hl)            ; read pointer
  inc l
  ld d,(hl)
  ex de,hl             ; hl -> sample data , de -> sccv.isrs_pnt high

  ; s w a p   i n   s a m p l e   d a t a

  ld (5000h),a         ; rom data from 4000h
  inc a                ; can comment out if 8k pages used for samples
  ld (7000h),a         ;

  rl c                 ; check sample replay mode
  jr c,.mode1

; t p   m o d e   0   -   i g n o r e   p e r i o d s 

.mode0            
                        ; note to myself :) use inc hl in sample data stream!!!
  inc hl                ; read period high only
  ld a,(hl)
  inc hl
  rlca                  ; check for jump or sample end
  jr c,.mode0_loop_or_stop

  ex de,hl              ; hl -> sccv.isrs_pnt high, de -> sample data
  ld (hl),d             ; save pointer, pointer now pointing to
  dec l                 ;               waveform for update
  ld (hl),e

  ld l,sccv.wave_update
  ld (hl),129           ; > 128 => wave update from isrs_pnt

  ld l,sccv.isrs_play
  xor a
  ret

.mode0_loop_or_stop
  rlca
  jr c,.stop

  ld c,(hl)             ; read sample loop offset
  inc hl
  ld b,(hl)
  dec hl
  dec hl
  dec hl                ; reset hl pnt to beginning of bin
  add hl,bc             ; add loop offset
  jr .mode0

.stop
  ex de,hl              ; hl -> sccv.isrs_pnt high
  ld l,sccv.isrs_play
  xor a
  ld (hl),a
  ret

; t p   m o d e   1   -   r e c a l c u l a t e    p e r i o d s 

.mode1

  ld c,(hl)             ; read period
  inc hl
  ld b,(hl)
  inc hl

  ld a,b
  rlca
  jr c,.mode1_loop_or_stop

  ex de,hl              ; hl -> sccv.isrs_pnt high, de -> sample data
  ld (hl),d             ; save pointer
  dec l
  ld (hl),e

  ld l,sccv.freq        ; read channel period (tp_target)
  ld e,(hl)
  inc l
  ld d,(hl)

  dec d                 ; test period with C-4 = 01AB
  jr nz,.recalc_tp      ; if C-4 take period as it is !!!
  ld a,0abh
  cp e
  jr nz,.recalc_tp
  inc d

.set_update_flags_and_return

  ld l,sccv.isrs_period
  ld d,129

  ld a,c                ; compare previous period
  cp (hl)
  jr nz,1f              ; period different => update period
  inc l
  ld a,b
  sub (hl)
  jr z,2f               ; period same as previous => no period update!

  dec l
1 ld (hl),c             ; save period
  inc l
  ld (hl),b
  ld a,d

2 ld l,sccv.freq_update
  ld (hl),a           ; > 128 => freq update from isrs_period
  ld l,sccv.wave_update
  ld (hl),d           ; > 128 => wave update from isrs_pnt

  ld l,sccv.isrs_play
  xor a
  ret

.recalc_tp
  inc d

  call shift_pitch_z80

  jp .set_update_flags_and_return

.mode1_loop_or_stop
  rlca
  jr c,.stop

  ld c,(hl)             ; read sample loop offset
  inc hl
  ld b,(hl)
  dec hl
  dec hl
  dec hl                ; reset hl pnt to beginning of bin
  add hl,bc             ; add loop offset
  jr .mode1



; R E C A L C U L A T E   S A M P L E   P E R I O D
;--------------------------------------------------------------------
;  in:  de = target_period  (12 bit)
;       bc = sample_period
;  ret: bc = new_period
;
;  tp_new = ( ( ((tp_target * 614) / 256) + 1 ) * tp_sample ) / 1024

shift_pitch_z80

  push hl       ; >>>
  push bc       ; >>>

  xor a         ; tp_target * 614  ( 10 0110 0110b )
  ld c,a        ; c = 0
  ld h,d        ; 1
  ld l,e
  add hl,hl     ; first four without overflow
  add hl,hl     ; 0
  add hl,hl     ; 0
  add hl,de     ; 1
  adc c
  add hl,hl
  rla
  add hl,de     ; 1
  adc c
  add hl,hl
  rla
  add hl,hl     ; 0
  rla
  add hl,hl     ; 0
  rla
  add hl,de     ; 1
  adc c  
  add hl,hl
  rla
  add hl,de     ; 1
  adc c  
  add hl,hl
  rla
  ld c,h        ; 0   last bit 0 so we are done
  ld b,a
  inc bc        ; bc = ((tp_target * 614) /256) + 1

  pop hl        ; <<< hl = sample period

  add hl,hl     ; 4 x pre-shift (align most significant bit 11 -> 15)
  add hl,hl
  add hl,hl
  add hl,hl
  ex de,hl      ; de = sample_period    

  ld hl,0       ; multiply de (12 bit) x bc (16 bit) = dehl (32 bit)
  sla e         ; first iteration
  rl d
  jr nc,1f
  ld h,b
  ld l,c
1
  repeat 11     ; remaining iterations, 12 - 1
  add hl,hl
  rl e 
  rl d
  jr nc,.jmp
  add hl,bc
  jr nc,.jmp
  inc de 
.jmp
  endrepeat

  ld a,e      ; 2x after-shift => / 1024
  ld c,h
  rra
  rr c
  rra
  rr c
  and 15
  ld b,a      ; bc = new_period

  pop hl      ; <<< restore hl
  ret
  
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

;shift_pitch_r800
;  push hl    ; >>>
;  push bc    ; >>>
;
;  ld l,e
;  ld h,d
;  ld bc,614
;  muluw hl,bc
;  ld l,h
;  ld h,e     
;  inc hl     ; shift factor = ( target_period * 614 ) / 256 + 1
;  
;  pop bc     ; <<< sample_period
;
;  muluw hl,bc
;
;  ld a,e     ; 2x after-shift
;  ld c,h
;  rra
;  rr c
;  rra
;  rr c
;  and 15
;  ld b,a     ; bc = new_period
;   
;  pop hl     ; <<<
;  xor a
;  ret


; T I M E R S  T A B L E
;--------------------------------------------------------------------
;

  code ##

@timertab
  byte 00001111b
  byte 00010011b
  byte 00101101b
  byte 01000111b
  byte 10011011b
  byte 00000111b
  byte 00101101b
  byte 00010011b
  byte 01001111b
  byte 00000111b
  byte 10111001b
  byte 00000111b
  byte 00001111b
  byte 01010011b
  byte 00101101b
  byte 00000111b
  byte 10011011b
  byte 00000111b
  byte 01101101b
  byte 00010011b
  byte 00001111b
  byte 00000111b
  byte 10111001b
  byte 01000111b
  byte 00001111b
  byte 00010011b
  byte 00101101b
  byte 00000111b
  byte 11011011b
  byte 00000111b
  byte 00101101b
  byte 00010011b
  byte 00001111b
  byte 01000111b
  byte 10111001b
  byte 00000111b
  byte 00001111b
  byte 00010011b
  byte 01101101b
  byte 00000111b
  byte 10011011b
  byte 00000111b
  byte 00101101b
  byte 01010011b
  byte 00001111b
  byte 00000111b
  byte 10111001b
  byte 00000111b
  byte 01001111b
  byte 00010011b
  byte 00101101b
  byte 00000111b
  byte 10011011b
  byte 01000111b
  byte 00101101b
  byte 00010011b
  byte 00001111b
  byte 00000111b
  byte 11111001b
  byte 00000111b

 
@timermasktab
  byte 1,2,4,8,16,32,64,128

  
  code

;--------------------------------------------------------------------

;eof
