; R E P L A Y E R
;-------------------------------------------------------------------
;                                                 XL2S Entertainment

; N E W
; I M P L E M E N T A T I E  V O O R  S A M P L E  P L A Y B A C K
;

; S C C    t o n e    s t r e a m
;--------------------------------------------------------------------
; in: h -> high sccv #
; uit:

; call scc_tone_stream.run 


scc_tone_stream

  ; p r o c e s s   I S R S   c o n t r o l  b y t e 
                                                      ; if no ISRS this can
                                                      ; be commented out
.isr_sample_control                                   ; 
  ld a,(de)          ; read ISRS control byte         ;
  inc de                                              ;
                                                      ;
  ld l,sccv.isrs_play                                 ;
  ld (hl),l            ; set isr sample playback on   ;
  inc l                                               ;
  ld (hl),255          ; first trigger => timer mask = 255 
  inc l                                               ;
  ld (hl),a            ; save copy of control byte    ;
  inc l                                               ;
  push hl              ; >>>                          ;
  rra                                                 ;
  and 111100b                                         ;
  ld hl,SAMPLE_JUMP_TABLE   ; format = 4 bytes / sam  ;
  add l                                               ;
  jr nc,1f                                            ;
  inc h                                               ;
1 ld l,a                                              ;
  ld a,(hl)            ; read page no. and pointer    ;
  ld (isr_samples_active),a       ; set active status ;
  inc l                                               ;
  ld c,(hl)                                           ;
  inc l                                               ;
  ld b,(hl)                                           ;
  pop hl               ; <<<                          ;
  ld (hl),a            ; hl -> .isrs_page             ;
  inc l                                               ;
  ld (hl),c            ; save pointer                 ;
  inc l                                               ;
  ld (hl),b                                           ;
  jp .save_stream_pointer


 ; c a l l   /   j u m p

.call
  ld b,a               ; hoge deel onthouden
  ld a,(de)            ; read low call adress
  inc de
  ld c,a               ; bc - call / jump adress

  ld l,sccv.call       ; in subroutine => jump i.p.v. call
  ld a,(hl)
  or a
  jr nz,.jump

  ld (hl),l            ; call active
  inc l                ; hl -> sccv.return  (save return adress)
  ld (hl),e
  inc l
  ld (hl),d

.jump
  ld a,64               ; shift adress range naar 4000h - bffffh
  add b
  ld e,c                ; de -> subroutine stroom 
  ld d,a
  jp .in_call_ret

  ; r e t u r n

.return
  ld l,sccv.call
  ld (hl),a            ; reset call status (a = 0 = ret code)
  inc l                ; hl -> sccv.return
  ld e,(hl)
  inc l
  ld d,(hl)
  jp .in_call_ret

  ; w a i t

.wait_set
  ld l,sccv.wait       ; hl -> wait
  dec a
  jr z,.wait_extented
.wait_ext_in
  dec a                ; aantal -1 (event zelf is al 1 wait)
  ld (hl),a
  jp .save_stream_pointer

.wait_extented
  ld a,(de)
  inc de
  jr .wait_ext_in

.wait_dec
  dec (hl)
  ret  

  ; r u n    s t r e a m                            < < < < < < < <

.run
  xor a

  ld l,sccv.wait       ; hl -> wait count
  cp (hl)
  jr nz,.wait_dec

  inc l                ; hl -> repeat count
  cp (hl)
  jr nz,.repeat 

  inc l                ; hl -> stroom pointer
  ld e,(hl)
  inc l
  ld d,(hl)            ; de -> stream
;  cp d
;  ret z                ; no stream check?

.in_call_ret

  ld a,(de)            ; read 1st byte from stream
  inc de

  or a
  jr z,.return
  rra                  ; 0 -> bit 7   (or a)
  jr c,.call
  rrca                 ; if set 1 or 2 then 1 -> bit 7 else 0 -> bit 7 
  jr c,.set_1or2       
  rra                  ; 0 -> bit 7 
  jr nc,.wait_set      

  rrca                 ; if sample then 1 -> bit 7 else 0 -> bit 7 
  jr c,.isr_sample_timer                       ; comment out if no ISRS

  ; s e t   0   -   v o l u m e 

.set_volume
  ld l,sccv.vol_update  ; update volume flag
  ld (hl),l
  inc l                 ; hl -> volume value
  ld (hl),a
  jp .save_stream_pointer

  ; i s r s   s t o p ,  t i m e r   m o d e  s w i t c h

.isr_sample_timer                              ; comment out if no ISRS
  and 15                                       ;
  ld l,sccv.isrs_play                          ; 
  jr z,.isr_stop_sample                        ;
  dec a                                        ;
  jr z,.isr_mode_switch                        ;
  dec a                                        ;
  jr z,1f               ; timer 0 = hold       ;
  ld b,h                ; keep h               ; 
  ld hl,timermasktab                           ;
  add l                                        ; 
  ld l,a                                       ; 
  ld a,(hl)             ; read timer mask      ;
  ld h,b                ; restore h            ;
1 ld l,sccv.isrs_timer                         ;
.isr_stop_sample                               ; 
  ld (hl),a             ; stop sample playback ;
  jr .in_call_ret       ; do next command      ; 
.isr_mode_switch                               ;
  ld l,sccv.isrs_control                       ;
  res 7,(hl)                                   ;
  jr .in_call_ret       ; do next command      ;

  ; s e t   1   o r   2

.set_1or2
  rrca                  ; if set 2 then 0 -> bit 7  else 1 -> bit 7
  jr nc,.set_relatief

  ; s e t   1   -   a b s o l u u t

.set_absoluut

  ld l,sccv.freq_update  ; update frequency flag
  ld (hl),l 
  inc l                ; hl -> frequency low
  ex de,hl
  ldi                  ; read frequency low from stream
  ex de,hl
  ld b,a               ; >>  keep for later           ; comment out if no ISRS 
  rra                  ; skip ISR bit
  and 15
  ld (hl),a            ; save frequency high

  inc l                ; hl -> update volume flag
  ld (hl),l            ; set volume update flag
  ld a,(de)            ; read volume byte from stream
  inc de
  inc l
  ld (hl),a            ; save volume

  rr b                 ; <<                           ; if no ISRS this can
  jp c,.isr_sample_control                            ; be commented out
                                                      ;
  ; s a v e   s t r e a m   p o i n t e r

.save_stream_pointer
  ld l,sccv.stream_pnt
  ld (hl),e
  inc l
  ld (hl),d
  ret

  ; s e t   2   -   r e l a t i e f

.rel_single_repeat
  ld (hl),1           ; set repeat 1x
  jr .rel_in_0

.rel_init_x_repeat
  ld a,(de)           ; read repeat count
  inc de
  dec a               ; kijken of dit anders kan!!!      * * * * * * 
  ld (hl),a
  ld l,sccv.ctrl_byte
  ld a,(hl)           ; read previous control byte
  jr .rel_in_1

.repeat
  dec (hl)            ; dec counter
  ld l,sccv.stream_pnt
  ld e,(hl)           ; kijken of dit anders kan!!!      * * * * * *      
  inc l
  ld d,(hl)
  ld l,sccv.ctrl_byte
  ld a,(hl)           ; read previous control byte
  jr .rel_in_1        ; execute command

.set_relatief                                      ; < < < < 

  ld l,sccv.repeat
  rrca
  jr c,.rel_single_repeat
  cp 20h                ; init multiple repeat
  jr z,.rel_init_x_repeat

.rel_in_0
  ld l,sccv.ctrl_byte
  ld (hl),a            ; save control byte for repeat

  ; r e l a t i v e   f r e q u e n c y

.rel_in_1
  rrca
  jr c,.rel_freq_add
  rrca
  ld b,a               ; >>
  jr nc,.rel_volume

.rel_set_value_and_add
  set 0,(hl)           ; adjust freq set value & add to just add

  ld a,(de)
  inc de
  inc l                ; hl -> deltafreq 
  ld (hl),a            ; save deltafreq byte

.rel_freq_add_in
  ld l,sccv.freq_update   ; hl -> update frequency flag
  ld (hl),l
  inc l                ; hl -> freq
  or a
  jp s,.rel_add_sign

  add (hl)
  ld (hl),a            ; update frequency low
  jr nc,.rel_volume
  inc l
  inc (hl)             ; update frequency high
  jr .rel_volume

.rel_add_sign
  add (hl)
  ld (hl),a            ; update frequency low
  jr c,.rel_volume     ; => c  i.p.v.  nc
  inc l
  dec (hl)             ; update frequency high
  jr .rel_volume

.rel_freq_add
  inc l                ; hl -> deltafreq 
  rrca
  ld b,a               ; >>
  ld a,(hl)            ; read deltafreq byte
  jr c,.rel_freq_add_in   ; add normal
  neg
  jr .rel_freq_add_in     ; add negative

  ; r e l a t i v e   v o l u m e

.rel_volume
  ld l,sccv.vol_update
  ld a,b               ; <<
  rrca
  jr c,.rel_inc_volume
  rrca
  jr nc,.save_stream_pointer

.rel_volume_add
  ld (hl),l            ; update volume flag
  inc l                ; hl -> volume
  ld a,(de)            ; read (signed) byte
  inc de
  add (hl)
  ld (hl),a            ; update volume
  jp .save_stream_pointer

.rel_inc_volume
  ld (hl),l            ; update volume flag
  inc l                ; hl -> volume
  rrca
  ld a,(hl)
  jr nc,.rel_inc_vol
  dec a
  ld (hl),a
  jp .save_stream_pointer
.rel_inc_vol
  inc a
  ld (hl),a
  jp .save_stream_pointer



;--------------------------------------------------------------------

;eof
