; R E P L A Y E R
;-------------------------------------------------------------------
;                                                 XL2S Entertainment

; S C C    w a v e    s t r e a m
;--------------------------------------------------------------------
; in: h -> high sccv #
; uit:

; call scc_wave_stream.run 


scc_wave_stream

.run
  xor a

  ld l,sccv.wstatus    ; hl -> wave processing status
  cp (hl)
  jr z,.waits          ; => next command

  ; i n c  /  d e c   w a v e

  ld a,(hl)

.exec_in
  ld l,sccv.wave_update
  ld (hl),l            ; set wave update
  inc l                ; hl -> wave pointer
  dec a
  jr nz,.dec_waves

  ; i n c   w a v e

.inc_wave
  ld a,32              ; inc wave pointer
  add (hl)
  ld (hl),a
  jr nc,1f
  inc l
  inc (hl)

1 ld l,sccv.incs      ; hl -> inc/dec counter
  dec (hl)            ; decrease counter
  ret nz
  dec l               ; hl -> wave processing status
  ld (hl),0           ; switch to waits
  ret

  ; d e c   w a v e 

.dec_waves
  ld a,-32            ; dec wave pointer
  add (hl)
  ld (hl),a
  jr c,1b
  inc l
  dec (hl)
  jr 1b

  ; w a i t s

.waits
  ld l,sccv.waits     ; (jr z => a = 0)
  cp (hl)
  jr z,.next_command
  dec (hl)
  ret

  ; n e x t   c o m m a n d

.next_command
  inc l                ; hl -> wave command repeat count
  cp (hl)
  jr z,.new_command

  ; r e p e a t    c o m m a n d

  dec (hl)             ; dec repeat counter
.repeat_in
  inc l                ; hl -> command byte
  ld a,(hl)
  jr 1f

.inc_or_dec_wave_cmd
  ld l,sccv.w_command
  ld (hl),a            ; save command for repeat
1 ld b,a
  cp 64
  ld c,1               ; status 1 = inc wave
  jr c,.init_inc_wave
  inc c                ; status 2 = dec wave
.init_inc_wave
  ld l,sccv.waits
  and 7
  ld (hl),a            ; init no. waits
  ld a,b
  rra
  rra
  rra
  and 7
  dec l                ; hl -> incs
  ld (hl),a            ; init no. incs / decs
  dec l                ; hl -> wave processing status
  ld a,c
  ld (hl),a            ; set porcessing status
  jp .exec_in

  ; n e w   c o m m a n d

.new_command
  ld l,sccv.w_stream_pnt + 1
  ld d,(hl)
  or d
  ret z                ; stream done => exit
  dec l
  ld e,(hl)            ; de -> wave stream

  ld a,(de)            ; read new command

  inc de               ; update & save stream pointer
  ld (hl),e
  inc l
  ld (hl),d

  or a
  jr z,.done
  jp ns,.inc_or_dec_wave_cmd

  bit 6,a
  jr nz,.set_wave
  bit 5,a
  jr z,.extended_wait

  ; i n i t   r e p e a t

  and 31
  ld l,sccv.w_repeat
  ld (hl),a            ; init no. repeats
  jp .repeat_in

  ; d o n e 

.done
  ld l,sccv.w_stream_pnt + 1
  ld (hl),a            ; wave updates done for this pattern
  ret

  ; e x t e n d e d   w a i t

.extended_wait
  and 31
  ld l,sccv.waits
  ld (hl),a            ; init no. waits
  ret

  ; s e t   w a v e

.set_wave
  bit 4,a
  jr z,.set_new_wave

  ; r e t r i g   w a v e

  ld l,sccv.waits
  and 15
  ld (hl),a            ; init no. waits

  ld l,sccv.wave_reset
  ld e,sccv.wave
  ld d,h
  ldi
  ldi
  ld l,sccv.wave_update
  ld (hl),l
  ret

.set_new_wave
  ld b,a               ;can be commented out if max 256 waves
  ld l,sccv.waits
  and 15
  ld (hl),a            ; init no. waits

  ld a,(de)            ; read new wave nummer from stream

  ld l,sccv.w_stream_pnt   ; update stream pointer
  inc (hl)
  jr nz,1f
  inc l
  inc (hl)
                       
1 ld e,0               ; calculate adress |--|8|76543| |210|-----|
  rra
  rr e
  rra
  rr e
  rra
  rr e
  and 31

  ld d,a               ; for 256 waves this can be commented out
  ld a,b               ; uncomment for 512 waves
  and 32               ;
  or d                 ; 

  ld d,a
  ld a,(wave_data_base + 1)
  add d                ; wave adress = ae

  ld l,sccv.wave_update
  ld (hl),l
  inc l                ; hl -> scc.wave
  ld (hl),e
  inc l
  ld (hl),a
  inc l                ; hl -> scc.wave_reset
  ld (hl),e
  inc l
  ld (hl),a
  ret


;--------------------------------------------------------------------

;eof
